package cn.ibizlab.codegen.templating;

import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * Locates generator-agnostic templates from a common built-in location.
 */
public class CommonTemplateContentLocator implements TemplatePathLocator {
    private String resourceLocation = "_common";

    /**
     * Constructs a new instance of {@link CommonTemplateContentLocator} defaulting to _common resource location.
     */
    public CommonTemplateContentLocator() {

    }

    /**
     * Constructs a new instance of {@link CommonTemplateContentLocator} for a targeted common location.
     *
     * @param resourceLocation A custom common file location.
     */
    public CommonTemplateContentLocator(String resourceLocation) {
        this.resourceLocation = resourceLocation;
    }

    public String getResourceLocation() {
        return resourceLocation;
    }

    /**
     * Get the full path to a relative template file.
     *
     * @param relativeTemplateFile Template file
     * @return String Full template file path
     */
    @Override
    public String getFullTemplatePath(String relativeTemplateFile) {
        if (StringUtils.isNotEmpty(relativeTemplateFile)) {
            String loc = Paths.get(this.resourceLocation , relativeTemplateFile).toString();

            URL url = this.getClass().getClassLoader().getResource(TemplateManager.getCPResourcePath(loc));
            if (url != null) {
                return loc;
            }
        }
        return null;
    }

    public List<String> getTemplatePaths() {
        List<String> list = new ArrayList<>();
        getList(this.resourceLocation,"",list,true);
        return list;
    }

    private void getList(String rootDir, String templateDir, List<String> list, boolean recu)  {
        try
        {
            if(templateDir.endsWith("//"))
                return;
            URL url = this.getClass().getClassLoader().getResource(rootDir+"/"+templateDir);
            if (url != null) {
                InputStream stream = this.getClass().getClassLoader().getResourceAsStream(rootDir+"/"+templateDir);
                try(Scanner scanner = new Scanner(stream)) {
                    while (scanner.hasNextLine()) {
                        String str = scanner.nextLine();
                        if(!org.springframework.util.StringUtils.isEmpty(str))
                        {
                            String newPath=str;
                            if(!org.springframework.util.StringUtils.isEmpty(templateDir))
                                newPath=templateDir+"/"+str;

                            if(recu&&(!str.endsWith(".hbs"))&&(!str.endsWith(".ibizlab-generator-ignore"))&&(str.indexOf(".")<0))
                                getList(rootDir,newPath,list,recu);
                            else
                                list.add(newPath);
                        }
                    }
                } catch (Exception e) {
                }
            }



        }catch (Exception ex){
            ex.printStackTrace();
        }

    }
}
